/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#ifndef _SM_INTER_H_RPCGEN
#define _SM_INTER_H_RPCGEN

#include <rpc/rpc.h>

/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user or with the express written consent of
 * Sun Microsystems, Inc.
 *
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 *
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 *
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 *
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 *
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */
/*
 * Copyright (c) 1986, 1990 by Sun Microsystems, Inc.
 */

/* from @(#)sm_inter.x	1.3 91/03/11 TIRPC 1.0 */

#ifndef _rpcsvc_sm_inter_h
#define _rpcsvc_sm_inter_h

#define SM_MAXSTRLEN 1024

struct sm_name {
	char *mon_name;
};
typedef struct sm_name sm_name;
#ifdef __cplusplus 
extern "C" bool_t xdr_sm_name(XDR *, sm_name*);
#elif __STDC__ 
extern  bool_t xdr_sm_name(XDR *, sm_name*);
#else /* Old Style C */ 
bool_t xdr_sm_name();
#endif /* Old Style C */ 


struct my_id {
	char *my_name;
	int my_prog;
	int my_vers;
	int my_proc;
};
typedef struct my_id my_id;
#ifdef __cplusplus 
extern "C" bool_t xdr_my_id(XDR *, my_id*);
#elif __STDC__ 
extern  bool_t xdr_my_id(XDR *, my_id*);
#else /* Old Style C */ 
bool_t xdr_my_id();
#endif /* Old Style C */ 


struct mon_id {
	char *mon_name;
	struct my_id my_id;
};
typedef struct mon_id mon_id;
#ifdef __cplusplus 
extern "C" bool_t xdr_mon_id(XDR *, mon_id*);
#elif __STDC__ 
extern  bool_t xdr_mon_id(XDR *, mon_id*);
#else /* Old Style C */ 
bool_t xdr_mon_id();
#endif /* Old Style C */ 


struct mon {
	struct mon_id mon_id;
	char priv[16];
};
typedef struct mon mon;
#ifdef __cplusplus 
extern "C" bool_t xdr_mon(XDR *, mon*);
#elif __STDC__ 
extern  bool_t xdr_mon(XDR *, mon*);
#else /* Old Style C */ 
bool_t xdr_mon();
#endif /* Old Style C */ 


struct sm_stat {
	int state;
};
typedef struct sm_stat sm_stat;
#ifdef __cplusplus 
extern "C" bool_t xdr_sm_stat(XDR *, sm_stat*);
#elif __STDC__ 
extern  bool_t xdr_sm_stat(XDR *, sm_stat*);
#else /* Old Style C */ 
bool_t xdr_sm_stat();
#endif /* Old Style C */ 


enum res {
	stat_succ = 0,
	stat_fail = 1,
};
typedef enum res res;
#ifdef __cplusplus 
extern "C" bool_t xdr_res(XDR *, res*);
#elif __STDC__ 
extern  bool_t xdr_res(XDR *, res*);
#else /* Old Style C */ 
bool_t xdr_res();
#endif /* Old Style C */ 


struct sm_stat_res {
	res res_stat;
	int state;
};
typedef struct sm_stat_res sm_stat_res;
#ifdef __cplusplus 
extern "C" bool_t xdr_sm_stat_res(XDR *, sm_stat_res*);
#elif __STDC__ 
extern  bool_t xdr_sm_stat_res(XDR *, sm_stat_res*);
#else /* Old Style C */ 
bool_t xdr_sm_stat_res();
#endif /* Old Style C */ 


struct status {
	char *mon_name;
	int state;
	char priv[16];
};
typedef struct status status;
#ifdef __cplusplus 
extern "C" bool_t xdr_status(XDR *, status*);
#elif __STDC__ 
extern  bool_t xdr_status(XDR *, status*);
#else /* Old Style C */ 
bool_t xdr_status();
#endif /* Old Style C */ 


#endif /*!_rpcsvc_sm_inter_h*/

#define SM_PROG ((u_long)100024)
#define SM_VERS ((u_long)1)

#ifdef __cplusplus
#define SM_STAT ((u_long)1)
extern "C" struct sm_stat_res * sm_stat_1(struct sm_name *, CLIENT *);
extern "C" struct sm_stat_res * sm_stat_1_svc(struct sm_name *, struct svc_req *);
#define SM_MON ((u_long)2)
extern "C" struct sm_stat_res * sm_mon_1(struct mon *, CLIENT *);
extern "C" struct sm_stat_res * sm_mon_1_svc(struct mon *, struct svc_req *);
#define SM_UNMON ((u_long)3)
extern "C" struct sm_stat * sm_unmon_1(struct mon_id *, CLIENT *);
extern "C" struct sm_stat * sm_unmon_1_svc(struct mon_id *, struct svc_req *);
#define SM_UNMON_ALL ((u_long)4)
extern "C" struct sm_stat * sm_unmon_all_1(struct my_id *, CLIENT *);
extern "C" struct sm_stat * sm_unmon_all_1_svc(struct my_id *, struct svc_req *);
#define SM_SIMU_CRASH ((u_long)5)
extern "C" void * sm_simu_crash_1(void *, CLIENT *);
extern "C" void * sm_simu_crash_1_svc(void *, struct svc_req *);

#elif __STDC__
#define SM_STAT ((u_long)1)
extern  struct sm_stat_res * sm_stat_1(struct sm_name *, CLIENT *);
extern  struct sm_stat_res * sm_stat_1_svc(struct sm_name *, struct svc_req *);
#define SM_MON ((u_long)2)
extern  struct sm_stat_res * sm_mon_1(struct mon *, CLIENT *);
extern  struct sm_stat_res * sm_mon_1_svc(struct mon *, struct svc_req *);
#define SM_UNMON ((u_long)3)
extern  struct sm_stat * sm_unmon_1(struct mon_id *, CLIENT *);
extern  struct sm_stat * sm_unmon_1_svc(struct mon_id *, struct svc_req *);
#define SM_UNMON_ALL ((u_long)4)
extern  struct sm_stat * sm_unmon_all_1(struct my_id *, CLIENT *);
extern  struct sm_stat * sm_unmon_all_1_svc(struct my_id *, struct svc_req *);
#define SM_SIMU_CRASH ((u_long)5)
extern  void * sm_simu_crash_1(void *, CLIENT *);
extern  void * sm_simu_crash_1_svc(void *, struct svc_req *);

#else /* Old Style C */ 
#define SM_STAT ((u_long)1)
extern  struct sm_stat_res * sm_stat_1();
extern  struct sm_stat_res * sm_stat_1_svc();
#define SM_MON ((u_long)2)
extern  struct sm_stat_res * sm_mon_1();
extern  struct sm_stat_res * sm_mon_1_svc();
#define SM_UNMON ((u_long)3)
extern  struct sm_stat * sm_unmon_1();
extern  struct sm_stat * sm_unmon_1_svc();
#define SM_UNMON_ALL ((u_long)4)
extern  struct sm_stat * sm_unmon_all_1();
extern  struct sm_stat * sm_unmon_all_1_svc();
#define SM_SIMU_CRASH ((u_long)5)
extern  void * sm_simu_crash_1();
extern  void * sm_simu_crash_1_svc();
#endif /* Old Style C */ 

#endif /* !_SM_INTER_H_RPCGEN */
